#!/usr/bin/env python

# Inspire by Conbagui tutorial found here: http://fav.me/damoiiu

# author: Pocholo
# date: 6/6/20

# Comments directed to http://gimpchat.com

# With the help of MareroQ 

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.


from gimpfu import *
import math

def pm_create_bottle_cap(customtext, fontname, size, justify, linespa, fontcolor, color):
	
	#Initiates the temporary state
	pdb.gimp_context_push()
	
	#Variables
	x = size/640
	
	#Create a new image
	img = pdb.gimp_image_new(size, size, RGB)
		
	#Create a background
	backLayer = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, backLayer, 0)
	pdb.gimp_layer_add_alpha(backLayer)
	pdb.gimp_drawable_fill(backLayer, FILL_WHITE)
	
	gimp.Display(img)
	
	#Making the cap effect
	capLayer = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Bottle cap", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, capLayer, 0)
	pdb.gimp_layer_add_alpha(capLayer)
	pdb.gimp_drawable_fill(capLayer, FILL_TRANSPARENT)
	
	#Used the Shape path - flowerspike
	pdb.script_fu_flowerspike_path(img, capLayer, 
                                        "Spikes",    #name 
                                           320*x,    #center x 
                                           320*x,    #center y 
                                           200*x,    #radius 
                                              24,    #number of side 
                                               0,    #rotation 
                                             0.7,    #factor A
                                               0,    #angular displacement A 
                                             0.7,    #factor B
                                               0,    #angular displacement B 
                                           False,    #stroke path 
                                 (255, 255, 255),    #stroke color 
                                            True,    #fill path 
                                 (188, 188, 188))    #fill color
		
	
	
	#Cutting the shape tips	
	pdb.gimp_image_select_ellipse(img, 2, 155*x, 155*x, 330*x, 330*x)
	pdb.gimp_selection_invert(img)
	pdb.gimp_edit_clear(capLayer)
	pdb.gimp_selection_invert(img)
	pdb.gimp_selection_none(img)
	
	#The cap drop shadow
	pdb.python_layerfx_drop_shadow(img, capLayer, 
                                       (0, 0, 0),   #color
                                              55,   #opacity
                                               0,   #contou
                                               0,   #noise 
                                               3,   #mode - multiply 
                                              50,   #spread 
                                            20*x,   #size 
                                             120,   #offset_angle 
                                               0,   #offset_distance 
                                            TRUE,   #knockout 
                                           FALSE)   #merge	
	
	#New layer keep shaping the cap 
	layer1 = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Layer #1", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, layer1, 0)
	pdb.gimp_layer_add_alpha(layer1)
	pdb.gimp_drawable_fill(layer1, FILL_TRANSPARENT)
		
	#Used the Shape path - star
	pdb.script_fu_star_path(img, layer1, 
                             "Spike #2",   #name
                                  320*x,    #center x 
                                  320*x,    #center y 
                                  200*x,    #outer radius  
                                  100*x,    #inner radius  
                                     24,    #number of point 
                                      0,    #rotation 
                                  False,    #stroke path 
                              (0, 0, 0),    #stroke color 
                                   True,    #fill path 
                        (135, 135, 135))    #fill color
	
	#Crown design layer
	layer2 = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Layer #2", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, layer2, 0)
	pdb.gimp_layer_add_alpha(layer2)
	pdb.gimp_drawable_fill(layer2, FILL_TRANSPARENT)
	
	#Make a selection and fill the crown selection
	Star = pdb.gimp_image_get_active_vectors(img)
	pdb.gimp_vectors_to_selection(Star, 2, TRUE, FALSE, 0, 0)
	pdb.gimp_selection_shrink(img, 3*x)
	pdb.gimp_context_set_foreground((144, 144, 144))
	pdb.gimp_edit_fill(layer2, FILL_FOREGROUND)
	
	#Cutting the tips of the star
	pdb.gimp_image_select_ellipse(img, 2, 155*x, 155*x, 330*x, 330*x)
	pdb.gimp_image_set_active_layer(img, layer1)
	pdb.gimp_selection_invert(img)
	pdb.gimp_edit_clear(layer1)
	pdb.gimp_image_set_active_layer(img, layer2)
	pdb.gimp_edit_clear(layer2)
	pdb.gimp_selection_none(img)
	
	#Cteating the top		
	layer3 = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Layer #3", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, layer3, 0)
	pdb.gimp_layer_add_alpha(layer3)
	pdb.gimp_drawable_fill(layer3, FILL_TRANSPARENT)
	pdb.gimp_image_select_ellipse(img, 2, 168*x, 168*x, 305*x, 305*x)
	pdb.gimp_context_set_foreground((128, 128, 128))
	pdb.gimp_edit_fill(layer3, FILL_FOREGROUND)
	pdb.gimp_selection_none(img)
	
	#Making the top with a little distress	
	layer4 = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Layer #3", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, layer4, 0)
	pdb.gimp_layer_add_alpha(layer4)
	pdb.gimp_drawable_fill(layer4, FILL_TRANSPARENT)
	pdb.gimp_image_select_ellipse(img, 2, 185*x, 185*x, 270*x, 270*x)
	pdb.script_fu_distress_selection(img, layer4, 
                                             127,   #threshold 
                                             4*x,   #spread 
                                             2*x,   #granuality 
                                             4*x,   #smooth 
                                            True,   #smooth horizontally 
                                            True)   #smooth vertically																					
	pdb.gimp_selection_feather(img, 12)
	pdb.gimp_context_set_foreground((185, 185, 185))
	pdb.gimp_edit_fill(layer4, FILL_FOREGROUND)
	pdb.gimp_selection_none(img)
	
	#Making the bottom layers invisible 
	shadow = pdb.gimp_image_get_layer_by_name(img, "Bottle cap-dropshadow")
	pdb.gimp_item_set_visible(backLayer, FALSE)	
	pdb.gimp_item_set_visible(shadow, FALSE)	
	
	#Create a new visible layer at the top of the pile
	caplayer2 = pdb.gimp_layer_new_from_visible(img, img, "Bottle crown cap")
	pdb.gimp_image_add_layer(img, caplayer2, 0)
	
	#Making the botom layers visible again and making the rest of the pile invisible
	pdb.gimp_item_set_visible(backLayer, TRUE)	
	pdb.gimp_item_set_visible(shadow, TRUE)
	pdb.gimp_item_set_visible(capLayer, FALSE)	
	pdb.gimp_item_set_visible(layer1, FALSE)	
	pdb.gimp_item_set_visible(layer2, FALSE)	
	pdb.gimp_item_set_visible(layer3, FALSE)
	pdb.gimp_item_set_visible(layer4, FALSE)
	
	#Apply color to the cap
	pdb.python_layerfx_color_overlay(img, caplayer2, 
                                            (color),   #color
                                                100,   #opacity
                                                  3,   #mode - multiply   
                                               True)   #merge
		
	#Duplicate the image for better look
	caplayer2 = pdb.gimp_image_get_layer_by_name(img, "Bottle crown cap")
	non_empty = pdb.gimp_edit_copy(caplayer2)
	caplayer2 = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Bottle crown cap copy", 100, LAYER_MODE_ADDITION)
	pdb.gimp_image_add_layer(img, caplayer2, 0)
	pdb.gimp_layer_add_alpha(caplayer2)
	pdb.gimp_drawable_fill(caplayer2, FILL_TRANSPARENT)
	floating_sel = pdb.gimp_edit_paste(caplayer2, FALSE)
	pdb.gimp_floating_sel_anchor(floating_sel)
				
	layer = pdb.gimp_layer_new(img, size, size, RGB, "Text", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, layer, 0)
	pdb.gimp_layer_add_alpha(layer)
	pdb.gimp_drawable_fill(layer, FILL_TRANSPARENT)
	pdb.gimp_context_set_foreground(fontcolor)

##############################################################################################################################

	#Making the text
	
	tex = size/3.3
	textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 10, 1, tex, PIXELS, fontname)
	pdb.gimp_text_layer_set_justification(textLayer, justify)
	pdb.gimp_text_layer_set_line_spacing(textLayer, linespa)
	
	pdb.gimp_layer_scale (textLayer, tex, tex, TRUE)
	
	
	#Making the text centered	
	textLayer = pdb.gimp_image_get_active_layer(img)
	pdb.gimp_image_set_active_layer(img, textLayer)
	wh = pdb.gimp_drawable_width(textLayer)
	hh = pdb.gimp_drawable_height(textLayer)
	w = pdb.gimp_image_width(img)
	h = pdb.gimp_image_height(img)
	pdb.gimp_image_set_active_layer(img,textLayer)
	centerx=w/2
	centery=h/2
	adw=wh/2
	adh=hh/2
	sw=centerx - adw
	sh=centery - adh		
	pdb.gimp_layer_set_offsets(textLayer, sw, sh)
	
	
	
	#Making the light reflection on a separate layer
	reflayer = pdb.gimp_layer_new(img, size, size, RGB_IMAGE, "Reflection", 30, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, reflayer, 0)
	pdb.gimp_layer_add_alpha(reflayer)
	pdb.gimp_drawable_fill(reflayer, FILL_TRANSPARENT)
	
	#Making the light reflection	
	pdb.gimp_image_select_ellipse(img, 2, 188*x, 188*x, 265*x, 265*x)
	pdb.gimp_edit_fill(reflayer, FILL_WHITE)
	pdb.gimp_image_select_ellipse(img, 2, 187*x, 165*x, 431*x, 375*x)
	pdb.gimp_edit_clear(reflayer)
	pdb.gimp_selection_none(img)
	pdb.plug_in_gauss(img, reflayer, 1.5, 1.5, 1)
			
	#Resets the previous user defaults
	pdb.gimp_context_pop()
	
register(
	"pm_create_bottle_cap",
	"Creates a bottle cap",
	"Creates abottle cap",
	"Pocholo",
	"Pocholo",
	"2020",
	"Bottle cap",
	"",
[	
	(PF_TEXT, "customtext", "Text string", 'Logo\nHere'),
	(PF_FONT, "font", "Font name", "Coca Cola ii"),
	(PF_ADJUSTMENT, "size", "Size", 640, (640, 1000, 1)),
	(PF_OPTION, "justify", "Justify", 2, ("Left", "Right", "Centered", "Fill")),
	(PF_ADJUSTMENT, "linespa", "Line spacing", 0, (-100, 100, 1)),
	(PF_COLOR, "fontcolor", "Font color", (0, 0, 0)),
	(PF_COLOR, "color", "Cap color", (220,0,0)),
],
[],
pm_create_bottle_cap, menu="<Image>/Pocholo-scripts/Bottle cap")
main()	